import flask
import storage
from flask import Flask
from flask_jwt_extended.exceptions import NoAuthorizationError
from flask_restplus import abort
from marshmallow.exceptions import ValidationError
from sqlalchemy.exc import SQLAlchemyError

from portal import logger
from portal.rest import rest_api
from portal.service.errors import IncorrectCredentialsError, PortalAPIError, UnauthorizedError

log = logger.get_logger(__name__)


def load_errors(app: Flask):
    log.trace("[LOAD] Custom error handlers loaded")
    for (ex, func) in rest_api.error_handlers.items():
        app.register_error_handler(ex, func)


@rest_api.errorhandler
def default_error_handler():
    return flask.jsonify({'message': 'Default error handler has been triggered'}), 401


@rest_api.errorhandler(UnauthorizedError)
def handle_unauthorized_error(ex: UnauthorizedError):
    log.warning(f"[AUTH] {ex.message} ")
    return flask.jsonify({'message': ex.message, 'error': f'{ex}'}), 401


@rest_api.errorhandler(NoAuthorizationError)
def handle_missing_auth_header(ex: NoAuthorizationError):
    log.warning(f"[AUTH] Auth headers are missing: {ex} ")
    return flask.jsonify({'message': 'Auth headers is missing', 'error': f'{ex}'}), 401


@rest_api.errorhandler(IncorrectCredentialsError)
def handle_missing_auth_header(ex: IncorrectCredentialsError):
    log.warning(f"[AUTH] Credentials are incorrect: {ex} ")
    return flask.jsonify({'message': 'Credentials are incorrect', 'error': f'{ex}'}), 401


@rest_api.errorhandler(ValidationError)
def handle_validation_error(ex: ValidationError):
    log.warning(f"[VALID] Validation failed: {ex}")
    return flask.jsonify({'message': f"Validation failed on: {ex.messages}"}), 400


@rest_api.errorhandler(SQLAlchemyError)
def handle_db_error(ex: SQLAlchemyError):
    log.error(f"[DB] Error: {ex}", ex)
    return flask.jsonify({'message': f'Database error: {ex}'}), 400


@rest_api.errorhandler(NotImplementedError)
def handle_not_implemented_error():
    log.warning(f"[WARN] Not implemented yet: {ex}")
    return flask.jsonify({'message': f'Not implemented yet!'}), 404


@rest_api.errorhandler(storage.errors.NotFoundError)
def handle_storage_not_found_error(ex: storage.errors.NotFoundError):
    log.warning(f"[STORAGE] Storage not found warning: {ex}")
    return flask.jsonify({'message': str(ex)}), 404


@rest_api.errorhandler(storage.errors.KontrStorageError)
def handle_storage_error(ex: storage.errors.KontrStorageError):
    log.warning(f"[STORAGE] Storage error: {ex}")
    return flask.jsonify({'message': str(ex)}), 400


@rest_api.errorhandler(PortalAPIError)
def handle_portal_api_error(ex: PortalAPIError):
    log.error(f"[API] Api error: {ex} ")
    abort(code=ex.code, message=ex.message)
